@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, Nokia
+ * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -21,8 +21,10 @@
 
 #include <sys/statvfs.h>
 #include <fcntl.h>
+#ifdef __linux__
 #include <sys/ioctl.h>
 #include <linux/msdos_fs.h>
+#endif /* __linux__ */
 #include <unistd.h>
 
 #include <glib/gi18n.h>
@@ -62,8 +64,8 @@ struct TrackerMinerFilesPrivate {
 
 	GVolumeMonitor *volume_monitor;
 
-        GSList *index_recursive_directories;
-        GSList *index_single_directories;
+	GSList *index_recursive_directories;
+	GSList *index_single_directories;
 
 	guint disk_space_check_id;
 	guint disk_space_pause_cookie;
@@ -150,6 +152,9 @@ static gboolean    miner_files_ignore_next_update_file  (TrackerMinerFS       *f
                                                          GFile                *file,
                                                          TrackerSparqlBuilder *sparql,
                                                          GCancellable         *cancellable);
+static void      extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
+                                                         ProcessFileData *data);
+
 
 G_DEFINE_TYPE (TrackerMinerFiles, tracker_miner_files, TRACKER_TYPE_MINER_FS)
 
@@ -546,6 +551,15 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			file = g_file_new_for_path (mount_point);
 			uri = g_file_get_uri (file);
 
+			g_string_append_printf (queries,
+			                        "DELETE FROM <%s> { "
+			                        "  <%s> tracker:mountPoint ?u "
+			                        "} WHERE { "
+			                        "  ?u a nfo:FileDataObject; "
+			                        "     nie:url \"%s\" "
+			                        "} ",
+			                        removable_device_urn, removable_device_urn, uri);
+
 			g_string_append_printf (queries,
 			                        "DROP GRAPH <%s> "
 			                        "INSERT INTO <%s> { "
@@ -555,7 +569,7 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			                        "  ?u a nfo:FileDataObject; "
 			                        "     nie:url \"%s\" "
 			                        "}",
-						removable_device_urn, removable_device_urn, removable_device_urn, uri);
+			                        removable_device_urn, removable_device_urn, removable_device_urn, uri);
 
 			g_object_unref (file);
 			g_free (uri);
@@ -1313,6 +1327,10 @@ miner_files_add_to_datasource (TrackerMinerFiles    *mf,
 static void
 process_file_data_free (ProcessFileData *data)
 {
+	g_signal_handlers_disconnect_by_func (data->cancellable,
+	                                      extractor_get_embedded_metadata_cancel,
+	                                      data);
+
 	g_object_unref (data->miner);
 	g_object_unref (data->sparql);
 	g_object_unref (data->cancellable);
@@ -1424,7 +1442,7 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 		                        "} WHERE { "
 		                        "  ?u a nfo:FileDataObject; "
 		                        "     nie:url \"%s\" "
-		                        "}",
+		                        "} ",
 		                        removable_device_urn, removable_device_urn, uri);
 
 		tracker_sparql_builder_append (data->sparql, queries->str);
@@ -1770,6 +1788,7 @@ tracker_miner_files_check_directory (GFile  *file,
 	 */
 	is_hidden = file_info && g_file_info_get_is_hidden (file_info);
 
+#ifdef __linux__
 	/* Second we check if the file is on FAT and if the hidden
 	 * attribute is set. GIO does this but ONLY on a Windows OS,
 	 * not for Windows files under a Linux OS, so we have to check
@@ -1789,6 +1808,7 @@ tracker_miner_files_check_directory (GFile  *file,
 			close (fd);
 		}
 	}
+#endif /* __linux__ */
 
 	if (is_hidden) {
 		/* FIXME: We need to check if the file is actually a
